﻿using System;
using System.Activities;
using System.Activities.Core.Presentation;
using System.Activities.DynamicUpdate;
using System.Activities.Presentation;
using System.Activities.Presentation.Services;
using System.Activities.XamlIntegration;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Reflection;
using System.Runtime.Serialization;
using System.Runtime.Versioning;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Xaml;
using Microsoft.Win32;

namespace Designer
{
    public partial class MainWindow : Window
    {
        private WorkflowDesigner designer;
        private ActivityBuilder activityBuilder;
        private bool updating = false;

        public MainWindow()
        {
            InitializeComponent();
        }

        protected override void OnInitialized(EventArgs e)
        {
            base.OnInitialized(e);

            (new DesignerMetadata()).Register();

            InitializeDesigner();
        }
        
        private void InitializeDesigner(string filename = null)
        {
            designer = new WorkflowDesigner();
            var cfgSvc = designer.Context.Services.GetService<DesignerConfigurationService>();
            cfgSvc.TargetFrameworkName = new FrameworkName(".NETFramework,Version=v4.5");

            DesignerBorder.Child = designer.View;
            PropertyBorder.Child = designer.PropertyInspectorView;

            if (String.IsNullOrEmpty(filename))
            {
                activityBuilder = new ActivityBuilder();
                designer.Load(activityBuilder);
                updating = false;
            }
            else
            {
                XamlXmlReaderSettings readerSettings = new XamlXmlReaderSettings()
                {
                    LocalAssembly = Assembly.GetExecutingAssembly()
                };

                XamlXmlReader xamlReader = new XamlXmlReader(filename, readerSettings);
                activityBuilder = XamlServices.Load(ActivityXamlServices.CreateBuilderReader(xamlReader)) as ActivityBuilder;
                DynamicUpdateServices.PrepareForUpdate(activityBuilder);

                designer.Load(activityBuilder);
                updating = true;
            }
        }

        private void OnNew(object sender, RoutedEventArgs e)
        {
            InitializeDesigner();
        }

        private void OnOpen(object sender, RoutedEventArgs e)
        {
            var dialog = new Microsoft.Win32.OpenFileDialog();
            dialog.FileName = "Workflow";
            dialog.DefaultExt = ".xaml";
            dialog.Filter = "XAML Files (.xaml)|*.xaml";

            if (dialog.ShowDialog() == true)
            {
                InitializeDesigner(dialog.FileName);
            }
        }

        private void OnSave(object sender, RoutedEventArgs e)
        {
            var dialog = new SaveFileDialog();
            dialog.InitialDirectory = @"C:\NTK\WF";
            dialog.FileName = "Workflow";
            dialog.DefaultExt = "xaml";
            dialog.Filter = "XAML Files (.xaml)|*.xaml";

            if (dialog.ShowDialog() == true)
            {
                designer.Flush();

                DynamicUpdateMap map = null;
                if (updating)
                {
                    map = DynamicUpdateServices.CreateUpdateMap(activityBuilder);
                }

                File.WriteAllText(dialog.FileName, designer.Text);

                if (map != null)
                {
                    DataContractSerializer serializer = new DataContractSerializer(typeof(DynamicUpdateMap));

                    using (FileStream fs = File.Open(System.IO.Path.ChangeExtension(dialog.FileName, ".map"), FileMode.Create))
                    {
                        serializer.WriteObject(fs, map);
                    }
                }
            }
        }
    }
}
