﻿using System;
using System.Activities;
using System.Activities.DurableInstancing;
using System.Activities.DynamicUpdate;
using System.Activities.XamlIntegration;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Runtime.DurableInstancing;
using System.Runtime.Serialization;
using System.Text;
using System.Threading;
using System.Threading.Tasks;

namespace DynamicUpdate
{
    class Program
    {
        private static readonly string path = @"C:\NTK\WF";

        static void Main(string[] args)
        {
            WorkflowIdentity v1Identity = new WorkflowIdentity
            {
                Name = "WF v1",
                Version = new Version(1, 0, 0, 0)
            };
            Activity v1Activity = LoadWorkflow("Workflow.1.xaml");

            WorkflowIdentity v2Identity = new WorkflowIdentity
            {
                Name = "WF v2",
                Version = new Version(2, 0, 0, 0)
            };
            Activity v2Activity = LoadWorkflow("Workflow.2.xaml");
            


            Console.WriteLine("Starting V1 instance...");

            WorkflowApplication application = new WorkflowApplication(v1Activity, v1Identity);
            Guid id = StartAndUnloadInstance(application);

            Console.WriteLine(String.Format("Instance persisted: {0}", id));
            Console.ReadLine();



            Console.WriteLine("Reading instance version: {0}...", id);

            WorkflowApplicationInstance instance = WorkflowApplication.GetInstance(id, GetInstanceStore());
            Console.WriteLine("Retrieved instance version information: {0}", instance.DefinitionIdentity.ToString());
            instance.Abandon();

            Console.ReadLine();



            Console.WriteLine("Resuming instance with V2 definition...");
            
            application = new WorkflowApplication(v2Activity, v2Identity);
            application.InstanceStore = GetInstanceStore();

            try
            {
                application.Load(id);
                ResumeBookmark(application);
            }
            catch (Exception e)
            {
                Console.WriteLine(e.Message);
            }

            Console.ReadLine();



            Console.WriteLine(String.Format("Updating workflow definition for {0}...", id));

            DynamicUpdateMap updateMap;
            using (FileStream fs = File.Open(Path.Combine(path, "Workflow.2.map"), FileMode.Open))
            {
                DataContractSerializer serializer = new DataContractSerializer(typeof(DynamicUpdateMap));
                updateMap = serializer.ReadObject(fs) as DynamicUpdateMap;
            }

            instance = WorkflowApplication.GetInstance(id, GetInstanceStore());

            application = new WorkflowApplication(v2Activity, v2Identity);
            application.Load(instance, updateMap);
            application.Unload();

            Console.WriteLine(String.Format("Workflow definition Updated for {0}", id));

            Console.ReadLine();
            


            Console.WriteLine("Reading instance version: {0}...", id);

            instance = WorkflowApplication.GetInstance(id, GetInstanceStore());
            instance.Abandon();
            Console.WriteLine("Retrieved instance version information: {0}", instance.DefinitionIdentity.ToString());

            Console.ReadLine();



            Console.WriteLine("Resuming instance with V2 definition...");

            application = new WorkflowApplication(v2Activity, v2Identity);
            application.InstanceStore = GetInstanceStore();

            application.Load(id);
            ResumeBookmark(application);

            Console.ReadLine();
        }

        private static Activity LoadWorkflow(string filename)
        {
            return ActivityXamlServices.Load(Path.Combine(path, filename));
        }

        public static Guid StartAndUnloadInstance(WorkflowApplication wfApplication)
        {
            AutoResetEvent syncEvent = new AutoResetEvent(false);
            wfApplication.PersistableIdle = (e) => PersistableIdleAction.Unload;
            wfApplication.Unloaded = (e) =>
            {
                syncEvent.Set();
            };

            wfApplication.InstanceStore = GetInstanceStore();
            wfApplication.Run();

            syncEvent.WaitOne();
            return wfApplication.Id;
        }

        public static void ResumeBookmark(WorkflowApplication wfApplication)
        {
            AutoResetEvent syncEvent = new AutoResetEvent(false);
            wfApplication.Unloaded = (e) => syncEvent.Set();

            wfApplication.ResumeBookmark("Wait", null);
            syncEvent.WaitOne();
        } 


        public static InstanceStore GetInstanceStore()
        {
            string connectionString = @"Data Source=.;Initial Catalog=SqlWorkflowInstanceStore;Integrated Security=False;User ID=wf;Password=wf";
            InstanceStore instanceStore = new SqlWorkflowInstanceStore(connectionString);

            InstanceHandle handle = instanceStore.CreateInstanceHandle();
            InstanceView view = instanceStore.Execute(handle, new CreateWorkflowOwnerCommand(), TimeSpan.FromSeconds(30));
            handle.Free();

            instanceStore.DefaultInstanceOwner = view.InstanceOwner;

            return instanceStore;
        }
    }
}
